

typedef _DISKINFO	DISKINFO;
typedef _DOSTIME	DOSTIME;
typedef _CCONLINE	CCONLINE;
typedef _DTA		DTA;
typedef _BPB		BPB;
typedef _MD			MD;
typedef _MPB		MPB;
typedef _IOREC		IOREC;
typedef _KBDVECS	KBDVECS;
typedef _KEYTAB		KEYTAB;


