/*
      PORTAB.H
*/


#if !defined( __PORTAB__ )
#define __PORTAB__

typedef          void    VOID;
typedef signed   char    BYTE;
typedef          short   WORD;
typedef unsigned short   UWORD;
typedef          long    LONG;
typedef unsigned char    UBYTE;
typedef unsigned long    ULONG;
typedef          int     BOOL;
typedef          int     BOOLEAN;

#define TRUE	(!0)
#define FALSE	0

#endif

/***********************************************************************/
